/*****************************************************************************
 *                                                                           *
 *  Copyright (c) 1993, 1994, Elan Feingold (elan@tasha.cheme.cornell.edu)   *
 *                                                                           *
 *     PERMISSION TO USE, COPY, MODIFY, AND TO DISTRIBUTE THIS SOFTWARE      *
 *     AND ITS DOCUMENTATION FOR ANY PURPOSE IS HEREBY GRANTED WITHOUT       *
 *     FEE, PROVIDED THAT THE ABOVE COPYRIGHT NOTICE APPEAR IN ALL           *
 *     COPIES AND MODIFIED COPIES AND THAT BOTH THAT COPYRIGHT NOTICE AND    *
 *     THIS PERMISSION NOTICE APPEAR IN SUPPORTING DOCUMENTATION.  THERE     *
 *     IS NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR      *
 *     ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS       *
 *     OR IMPLIED WARRANTY.                                                  *
 *                                                                           *
 *****************************************************************************/

#include "types.h"
#include "debug.h"
#include "thread.h"
#include "factory.hh"
#include "blob.hh"

extern Factory factory;

void blob_thread(any_ptr pvState);


/************************************************************************ 
 *  FUNCTION: Blob::Start
 *  HISTORY: 
 *     11/16/93  ESF  Created 
 *  PURPOSE: 
 *     Simply forks off a blob thread.  A Blob is the object that 
 *   travels through a Factory object and gets transformed. 
 *  NOTES: 
 ************************************************************************/
void Blob::Start(String strName)
{
  t_forkRT((ThreadFunc)blob_thread, (any_ptr)strName, strName, PRIORITY_LOW);
}


/************************************************************************ 
 *  FUNCTION: blob_thread 
 *  HISTORY: 
 *     11/16/93  ESF  Created 
 *  PURPOSE:
 *     This is the function that moves the blob through the factory 
 *   object.  for more information, see the Factory object.  
 *  NOTES: 
 ************************************************************************/
void blob_thread(any_ptr strName)
{
  UInt iNumStages, iNumSteps, i, j;
  Int  iID, iNewID;

  /* How many stages are there? */
  iNumStages = factory.QueryStages();

  /* Start by getting the first resource */
  iID = factory.AcquireResource(0, 0, 0);

  for(i=0; i!=iNumStages; i++)
    for(j=0, iNumSteps=factory.QuerySteps(i); j!=iNumSteps; j++, iID=iNewID)
      {
	/* Actually use the resource (may take time) */
	factory.UseResource(i, j, iID);
	
	/* Before we release a resource, acquire the next */
	iNewID = factory.AcquireResource(NextStage(factory, i, j), 
					 NextStep(factory, i, j), iID);
	
	/* Now release the resource */
	factory.ReleaseResource(i, j, iID);
      }
}


















