/*****************************************************************************
 *                                                                           *
 *  Copyright (c) 1993, 1994, Elan Feingold (elan@tasha.cheme.cornell.edu)   *
 *                                                                           *
 *     PERMISSION TO USE, COPY, MODIFY, AND TO DISTRIBUTE THIS SOFTWARE      *
 *     AND ITS DOCUMENTATION FOR ANY PURPOSE IS HEREBY GRANTED WITHOUT       *
 *     FEE, PROVIDED THAT THE ABOVE COPYRIGHT NOTICE APPEAR IN ALL           *
 *     COPIES AND MODIFIED COPIES AND THAT BOTH THAT COPYRIGHT NOTICE AND    *
 *     THIS PERMISSION NOTICE APPEAR IN SUPPORTING DOCUMENTATION.  THERE     *
 *     IS NO REPRESENTATIONS ABOUT THE SUITABILITY OF THIS SOFTWARE FOR      *
 *     ANY PURPOSE.  THIS SOFTWARE IS PROVIDED "AS IS" WITHOUT EXPRESS       *
 *     OR IMPLIED WARRANTY.                                                  *
 *                                                                           *
 *****************************************************************************/

#include <stdlib.h>
#include "types.h"
#include "thread.h"
#include "debug.h"
#include "factory.hh"
#include "callbacks.hh"
#include "bitmaps.hh"
#include "x.hh"

/* Local globals */
Factory  factory(640, 480);

/* Local prototypes */
void SpawnBlobs(any_ptr pvFoo);
void BlobFunc(any_ptr pvFoo);

/************************************************************************ 
 *  FUNCTION: main
 *  HISTORY: 
 *     11/16/93  ESF  Created
 *  PURPOSE:
 *     Creates the factory, and then starts everything rolling by
 *   calling Factory::Start().
 *  NOTES: 
 *     For more information on the methods used to create the factory,
 *   see factory.C, and factory.hh.
 ************************************************************************/
main(int argc, char **argv)
{
  Int iStage;

  /*******************************************************************/
  iStage = factory.StageAdd("Conveyor Belt #1", 1, 5, 1, Millisec(500000));
  factory.StageRegisterCoords(iStage, 30, 75);
  factory.StageRegisterCallback(iStage, CALLBACK_LIFE | CALLBACK_CLICK, 
				CB_ConveyorBelt);
  factory.StageRegisterCallback(iStage, CALLBACK_COORDSERVER, 
				CB_ConveyorBeltCoordServer);
  factory.StageRegisterCallback(iStage, CALLBACK_ACQUIRE | CALLBACK_RELEASE,
				CB_Blob);

  /*******************************************************************/
  iStage = factory.StageAdd("Deluxe Mixer", 1, 1, 11, Millisec(250000));
  factory.StageRegisterCoords(iStage, 450, 10);
  factory.StageRegisterCallback(iStage, CALLBACK_LIFE | CALLBACK_CLICK, 
				CB_Mixer);

  /*******************************************************************/
  iStage = factory.StageAdd("Conveyor Belt #2", 2, 5, 1, Millisec(1400000));
  factory.StageRegisterCoords(iStage, 30, 185);
  factory.StageRegisterCallback(iStage, CALLBACK_LIFE | CALLBACK_CLICK, 
				CB_ConveyorBelt);
  factory.StageRegisterCallback(iStage, CALLBACK_COORDSERVER, 
				CB_ConveyorBeltCoordServer);
  factory.StageRegisterCallback(iStage, CALLBACK_ACQUIRE | CALLBACK_RELEASE,
				CB_Blob);

  /*******************************************************************/
  iStage = factory.StageAdd("Oven", 1, 1, 4, Millisec(1000000));
  factory.StageRegisterCoords(iStage, 460, 195);
  factory.StageRegisterCallback(iStage, CALLBACK_LIFE | CALLBACK_CLICK, 
				CB_Oven);

  /*******************************************************************/
  iStage = factory.StageAdd("Conveyor Belt #3", 1, 5, 1, Millisec(500000));
  factory.StageRegisterCoords(iStage, 30, 400);
  factory.StageRegisterCallback(iStage, CALLBACK_LIFE | CALLBACK_CLICK, 
				CB_ConveyorBelt);
  factory.StageRegisterCallback(iStage, CALLBACK_COORDSERVER, 
				CB_ConveyorBeltCoordServer);
  factory.StageRegisterCallback(iStage, CALLBACK_ACQUIRE | CALLBACK_RELEASE,
				CB_Blob);

  /*******************************************************************/
  iStage = factory.StageAdd("Truck", 5, 1, 0, 0);
  factory.StageRegisterCoords(iStage, 450, 360);
  factory.StageRegisterCallback(iStage, CALLBACK_LIFE | CALLBACK_CLICK |
				CALLBACK_FINISHED, CB_Truck);

  factory.RegisterSpawnFunction((ThreadFunc)SpawnBlobs);
  factory.Start(argc, argv);
}


/************************************************************************ 
 *  FUNCTION: SpawnBlobs
 *  HISTORY: 
 *     11/21/93  ESF  Created
 *  PURPOSE:
 *     This function starts a blob every certain interval to travel
 *   through the factory.
 *  NOTES: 
 ************************************************************************/
void SpawnBlobs(any_ptr pvFoo)
{
  PCondition  pCond = cond_create();
  Int         i;
  char        strName[32];

  cond_registerRT(pCond, Millisec(1300000));

  for(i=0;;i++)
    {
      sprintf(strName, "Blob[%d]", i);
      t_forkRT((ThreadFunc)BlobFunc, NULL, strName, PRIORITY_LOW);
      t_wait(pCond);
    }
}


/************************************************************************ 
 *  FUNCTION: BlobFunc
 *  HISTORY: 
 *     11/16/93  ESF  Created 
 *  PURPOSE:
 *     This is the function that moves the blob through the factory 
 *   object.  for more information, see the Factory object.  
 *  NOTES: 
 ************************************************************************/
void BlobFunc(any_ptr pvFoo)
{
  UInt iNumStages, iNumSteps, i, j;
  Int  iID, iNewID;

  /* How many stages are there? */
  iNumStages = factory.QueryStages();

  /* Start by getting the first resource */
  iID = factory.AcquireResource(0, 0, 0);

  for(i=0; i!=iNumStages; i++)
    for(j=0, iNumSteps=factory.QuerySteps(i); j!=iNumSteps; j++, iID=iNewID)
      {
	/* Actually use the resource (may take time) */
	factory.UseResource(i, j, iID);
	
	/* Before we release a resource, acquire the next */
	iNewID = factory.AcquireResource(NextStage(factory, i, j), 
					 NextStep(factory, i, j), iID);
	
	/* Now release the resource */
	factory.ReleaseResource(i, j, iID);
      }
}

